def factorial(x):
    from .exp import exp
    from .integrate import integrate
    if type(x) is not complex:
        if int(x) == x:
            if x < 0:
                raise ValueError("Factorial of a negative integer is undefined.")
            else:
                ans = 1
                for i in range(1,x+1):
                    ans *= i
                return ans
    else:
        def integrand(t):
            return t**x * exp(-t)
        return integrate(integrand, 0, 100)



def quad(a,b,c):
    from .sqrt import sqrt
    if a == 0:
        raise ValueError("value of 'a' must be different to 0.")
    if a+b+c == 0:
        quad.x1 = 1
        quad.x2 = c/a
    elif a-b+c == 0:
        quad.x1 = -1
        quad.x2 = -c/a
    else:
        delta = b**2-4*a*c
        quad.x1 = (-b+sqrt(delta))/(2*a)
        quad.x2 = (-b-sqrt(delta))/(2*a)
    pre_result = [quad.x1, quad.x2]
    result = []
    for solution in pre_result:
        if solution not in result:
            result.append(solution)
    return result


def among_us(PlayersNumber: int, lang="eng"):
    if type(PlayersNumber) != int:
        raise TypeError
    elif PlayersNumber <= 2:
        print("Players number should be greater than 2")
    else:
        from random import choice
        if lang == "eng":
            players = {}
            words = ["Table", "Chair", "Lamp", "Sofa", "Bookshelf", "Television", "Remote control", "Clock", "Mirror", "Vase", 
                "Pillow", "Blanket", "Curtain", "Rug", "Desk", "Computer", "Mouse", "Keyboard", "Monitor", "Printer", 
                "Refrigerator", "Stove", "Microwave", "Toaster", "Dishwasher", "Plate", "Fork", "Knife", "Spoon", "Cup", 
                "Vase", "Plant", "Picture frame", "Candle", "Basket", "Tray", "Towel", "Soap", "Shampoo", "Toothbrush", 
                "Toothpaste", "Towel rack", "Toilet", "Sink", "Mirror", "Soap dispenser", "Towel holder", "Shower", "Bathtub", 
                "Bed", "Mattress", "Pillowcase", "Blanket", "Alarm clock", "Dresser", "Wardrobe", "Hanger", "Shoes", 
                "Slippers", "Closet", "Coat rack", "Umbrella", "Hat", "Gloves", "Scarf", "Sunglasses", "Wallet", 
                "Keychain", "Purse", "Backpack", "Suitcase", "Briefcase", "Pen", "Pencil", "Notebook", "Eraser", 
                "Stapler", "Tape", "Scissors", "Glue", "Calculator", "Ruler", "Folder", "Paperclip", "Document", 
                "Envelope", "Stamp", "Trash can", "Recycling bin", "Laundry basket", "Iron", "Ironing board", 
                "Detergent", "Broom", "Dustpan", "Mop", "Bucket", "Vacuum cleaner", "Cleaning cloth", "Air freshener",
                "Sofa", "Television", "Heater", "Fan", "Air conditioner", "Bookshelf", "Console", "Speaker", "Book", 
                "Newspaper", "Magazine", "CD", "DVD", "Video game", "Camera", "Camcorder", "Phone", "Tablet", "Headphones", 
                "Charger", "Battery", "Cable", "Wall clock", "Pocket watch", "Wristwatch", "Jewelry", "Wallet", 
                "Card", "Paper", "Fountain pen", "Brush", "Paint", "Color pencil", "Brush", "Canvas", "Easel", 
                "Ink", "Shelf", "Library", "Chest of drawers", "Makeup mirror", "Perfume", "Jewelry box", 
                "Cushion", "Throw", "Tablecloth", "Dishware", "Cutlery", "Kitchen", "Glass", "Coffee cup", "Tea cup", 
                "Plate", "Bowl", "Tablecloth", "Napkin", "Shower curtain", "Heated towel rack", "Toilet", 
                "Sink", "Bathtub", "Shower", "Towel", "Bathrobe", "Hair dryer", "Hair straightener", "Hairbrush", 
                "Makeup", "Eyeshadow", "Lipstick", "Mascara", "Makeup brush", "Cotton swab", "Makeup remover", 
                "Moisturizer", "Deodorant", "Shampoo", "Conditioner", "Shower gel", "Soap", "Lotion", "Lip balm", 
                "Tissue", "Trash bag", "Laundry basket", "Laundry", "Fabric softener", "Ironing board", "Iron", 
                "Detergent", "Broom", "Dustpan", "Bucket", "Cloth", "Vacuum cleaner", "Brush", "Hanger", "Coat rack", 
                "Shoe rack", "Welcome mat", "Doormat", "Umbrella", "Cap", "Hat", "Sunglasses", "Scarf", "Gloves", 
                "Watch", "Necklace", "Bracelet", "Ring", "Wallet", "Cardholder", "Key", "Phone", "Keychain", 
                "Desk lamp", "Flashlight", "Table lamp", "Chandelier", "Light bulb", "Picture frame", "Wall mirror", 
                "Statue", "Plant", "Flower", "Vase", "Photograph", "Painting", "Sculpture", "Wall clock", "Alarm clock", 
                "Pendulum", "Watch", "Socks", "Tights", "Scarf", "Beanie", "Gloves", "Pajamas", "Nightgown", "Bathrobe", 
                "Towel", "Bath mat", "Scale", "Toothbrush", "Toothpaste", "Dental floss", "Tissues", "Toilet paper", 
                "Toilet brush", "Deodorant", "Shampoo", "Conditioner", "Shower gel", "Soap", "Moisturizer", "Lip balm", 
                "Tissues", "Waste bin", "Laundry basket", "Laundry detergent", "Fabric softener", "Ironing board", 
                "Iron", "Detergent", "Broom", "Dustpan", "Bucket", "Cloth", "Vacuum cleaner", "Brush", "Hanger", 
                "Coat rack", "Shoe rack", "Welcome mat", "Doormat", "Umbrella", "Cap", "Hat", "Sunglasses", 
                "Scarf", "Gloves", "Watch", "Necklace", "Bracelet", "Ring", "Wallet", "Cardholder", "Key", "Phone", 
                "Keychain"]
            randomize = choice(words)
            word = [randomize] * (PlayersNumber-1)
            word.append("imposter")
            for i in range(PlayersNumber):
                player = input("Enter your name:\n>>    ")
                players[i] = player
                identity = choice(word)
                if identity == "imposter":
                    imposter = i
                    word.remove("imposter")
                    pass_the_phone = input("\033[91mYOU ARE THE IMPOSTER\n\033[0mPress enter and pass the phone to another player: ")
                    print("\n" * 4000)
                else:
                    pass_the_phone = input(f"\033[92mYOU ARE A CREWMATE\n\033[0mThe word is {identity}\nPress enter and pass the phone to another player: ")
                    word.remove(word[0])
                    print("\n" * 4000)
            while len(players) > 2 and imposter in players:
                start_voting = input("Press enter when you are going to start voting for the imposter: ")
                print(players, "\n Each Player vote to what he want using player's id!")
                votes = {}
                for i in range(len(players)):
                    id = int(input("\nWhich Player you are going to vote?\n>>    "))
                    if id in votes:
                        votes[id] += 1
                    else:
                        votes[id] = 1
                out = max(votes, key=votes.get)
                found_duplicate = False
                for key1, value1 in votes.items():
                    for key2, value2 in votes.items():
                        if key1 != key2 and value1 == value2:
                            found_duplicate = True
                            print("No player has been kicked from the game")
                            break
                    if found_duplicate:
                        break
                if not found_duplicate:
                    print(players[out], "was kicked from the game")
                    if out == imposter:
                        print(players[out], "is the imposter\n\033[92mCrewmates won!")
                        break
                    else:
                        print(players[out], "is not the imposter")
                        players.pop(out)
            else:
                print(players[imposter], "was the imposter\n\033[91mImposter won!")
        elif lang == "fr":
            players = {}
            words = ["Table", "Chaise", "Lampe", "Canapé", "Étagère", "Télévision", "Télécommande", "Horloge", "Miroir", "Vase", 
                "Oreiller", "Couverture", "Rideau", "Tapis", "Bureau", "Ordinateur", "Souris", "Clavier", "Moniteur", 
                "Imprimante", "Réfrigérateur", "Cuisinière", "Micro-ondes", "Grille-pain", "Lave-vaisselle", "Assiette", 
                "Fourchette", "Couteau", "Cuillère", "Tasse", "Plante", "Cadre photo", "Bougie", "Panier", "Plateau", 
                "Serviette", "Savon", "Shampoing", "Brosse à dents", "Dentifrice", "Porte-serviettes", "Toilette", "Évier", 
                "Distributeur de savon", "Porte-serviettes", "Douche", "Baignoire", "Lit", "Matelas", "Taie d'oreiller", 
                "Dresser", "Armoire", "Cintre", "Chaussures", "Chaussons", "Placard", "Porte-manteau", "Parapluie", "Chapeau", 
                "Gants", "Écharpe", "Lunettes de soleil", "Portefeuille", "Porte-clés", "Sac à main", "Sac à dos", "Valise", 
                "Mallette", "Stylo", "Crayon", "Cahier", "Gomme", "Agrafeuse", "Ruban adhésif", "Ciseaux", "Colle", "Calculatrice", 
                "Règle", "Dossier", "Trombone", "Document", "Enveloppe", "Timbre", "Poubelle", "Poubelle de recyclage", 
                "Panier à linge", "Fer à repasser", "Planche à repasser", "Détergent", "Balai", "Pelle à poussière", 
                "Serpillière", "Seau", "Aspirateur", "Chiffon de nettoyage", "Assainisseur d'air", "Canapé", "Télévision", 
                "Radiateur", "Ventilateur", "Climatiseur", "Étagère à livres", "Console", "Haut-parleur", "Livre", "Journal", 
                "Magazine", "CD", "DVD", "Jeu vidéo", "Appareil photo", "Caméra", "Téléphone", "Tablette", "Casque", "Chargeur", 
                "Batterie", "Câble", "Horloge murale", "Horloge de poche", "Montre", "Bijoux", "Porte-monnaie", "Carte", 
                "Papier", "Stylo-plume", "Pinceau", "Peinture", "Crayon de couleur", "Pinceau", "Toile", "Chevalet", 
                "Encre", "Étagère", "Bibliothèque", "Commode", "Miroir de maquillage", "Parfum", "Boîte à bijoux", 
                "Coussin", "Jeté", "Nappe", "Vaisselle", "Couverts", "Cuisine", "Verre", "Tasse à café", "Tasse à thé", 
                "Plat", "Bol", "Nappe", "Serviette de table", "Rideau de douche", "Porte-serviettes chauffant", "Toilette", 
                "Lavabo", "Baignoire", "Douche", "Serviette", "Peignoir", "Sèche-cheveux", "Lisseur", "Brosse à cheveux", 
                "Maquillage", "Fard à paupières", "Rouge à lèvres", "Mascara", "Pinceau de maquillage", "Coton-tige", 
                "Démaquillant", "Crème hydratante", "Parapluie", "Pantalon", "Chemise", "Robe", "Chaussures", "Sac", 
                "Chapeau", "Gants", "Écharpe", "Lunettes de soleil", "Ceinture", "Montre", "Boucle d'oreille", "Collier", 
                "Bracelet", "Bague", "Portefeuille", "Porte-cartes", "Clé", "Téléphone", "Porte-clés", "Lampe de bureau", 
                "Lampe de poche", "Lampe de table", "Lustre", "Ampoule", "Cadre photo", "Miroir mural", "Statue", "Plante", 
                "Fleur", "Vase", "Photographie", "Peinture", "Sculpture", "Horloge murale", "Réveil", "Pendule", "Montre", 
                "Chaussettes", "Collants", "Écharpe", "Bonnet", "Gants", "Pyjama", "Chemise de nuit", "Peignoir", "Serviette", 
                "Tapis de bain", "Pèse-personne", "Brosse à dents", "Dentifrice", "Fil dentaire", "Mouchoirs", "Papier toilette", 
                "Brosse à toilette", "Déodorant", "Shampooing", "Après-shampooing", "Gel douche", "Savon", "Crème hydratante", 
                "Baume à lèvres", "Mouchoirs en papier", "Sac poubelle", "Panier à linge", "Lessive", "Adoucissant", "Planche à repasser", 
                "Fer à repasser", "Détergent", "Balai", "Pelle à poussière", "Seau", "Chiffon", "Aspirateur", "Brosse", "Cintre", 
                "Porte-manteau", "Étagère à chaussures", "Tapis d'entrée", "Paillasson", "Parapluie", "Casquette", "Chapeau", "Lunettes de soleil", 
                "Écharpe", "Gants", "Montre", "Collier", "Bracelet", "Bague", "Portefeuille", "Sac à main", "Sac à dos", "Valise", "Mallette"]
            randomize = choice(words)
            word = [randomize] * (PlayersNumber-1)
            word.append("imposter")
            for i in range(PlayersNumber):
                player = input("Enter your name:\n>>    ")
                players[i] = player
                identity = choice(word)
                if identity == "imposter":
                    imposter = i
                    word.remove("imposter")
                    pass_the_phone = input("\033[91mYOU ARE THE IMPOSTER\n\033[0mPress enter and pass the phone to another player: ")
                    print("\n" * 4000)
                else:
                    pass_the_phone = input(f"\033[92mYOU ARE A CREWMATE\n\033[0mThe word is {identity}\nPress enter and pass the phone to another player: ")
                    word.remove(word[0])
                    print("\n" * 4000)
            while len(players) > 2 and imposter in players:
                start_voting = input("Press enter when you are going to start voting for the imposter: ")
                print(players, "\n Each Player vote to what he want using player's id!")
                votes = {}
                for i in range(len(players)):
                    id = int(input("\nWhich Player you are going to vote?\n>>    "))
                    if id in votes:
                        votes[id] += 1
                    else:
                        votes[id] = 1
                out = max(votes, key=votes.get)
                found_duplicate = False
                for key1, value1 in votes.items():
                    for key2, value2 in votes.items():
                        if key1 != key2 and value1 == value2:
                            found_duplicate = True
                            print("No player has been kicked from the game")
                            break
                    if found_duplicate:
                        break
                if not found_duplicate:
                    print(players[out], "was kicked from the game")
                    if out == imposter:
                        print(players[out], "is the imposter\n\033[92mCrewmates won!")
                        break
                    else:
                        print(players[out], "is not the imposter")
                        players.pop(out)
            else:
                print(players[imposter], "was the imposter\n\033[91mImposter won!")
        elif lang == "ar":
            players = {}
            words = [
                "طاولة", "كرسي", "مصباح", "صوفا", "رف الكتب", "تلفاز", "جهاز التحكم عن بعد", "ساعة", "مرآة", "زهرية",
                "وسادة", "بطانية", "ستارة", "سجادة", "مكتب", "كمبيوتر", "فأرة", "لوحة المفاتيح", "شاشة العرض", "طابعة",
                "ثلاجة", "موقد", "ميكروويف", "محمصة", "غسالة صحون", "صحن", "شوكة", "سكين", "ملعقة", "كوب",
                "زهرية", "نبات", "إطار الصورة", "شمعة", "سلة", "صينية", "منشفة", "صابون", "شامبو", "فرشاة الأسنان",
                "معجون الأسنان", "رف المناشف", "مرحاض", "حوض", "مرآة", "موزع الصابون", "حامل المناشف", "دش", "حوض الاستحمام",
                "سرير", "فراش", "وسادة", "بطانية", "منبه", "خزانة الملابس", "خزانة", "شماعة", "أحذية",
                "شباشب", "خزانة", "رف معاطف", "مظلة", "قبعة", "قفازات", "وشاح", "نظارات شمسية", "محفظة",
                "سلسلة المفاتيح", "محفظة", "حقيبة ظهر", "حقيبة", "حقيبة", "قلم", "قلم رصاص", "دفتر", "ممحاة",
                "مدبب", "شريط", "مقص", "غراء", "آلة حاسبة", "مسطرة", "مجلد", "مشبك ورق", "وثيقة",
                "ظرف", "طابعة", "سلة المهملات", "صندوق إعادة التدوير", "سلة الملابس", "مكواة", "مكتب المكواة",
                "منظف", "مكنسة", "جارف الغبار", "ممسحة", "دلو", "مكنسة كهربائية", "قماش تنظيف", "معطر الهواء",
                "صوفا", "تلفزيون", "سخان", "مروحة", "مكيف هواء", "رف الكتب", "جهاز اللعب", "مكتب", "كتاب",
                "جريدة", "مجلة", "قرص مدمج", "دي في دي", "لعبة فيديو", "كاميرا", "كاميرا فيديو", "هاتف", "جهاز لوحي",
                "سماعات", "شاحن", "بطارية", "كابل", "ساعة حائط", "ساعة جيب", "ساعة يد", "مجوهرات", "محفظة",
                "بطاقة", "ورق", "قلم حبر", "فرشاة", "طلاء", "ألوان ملونة", "فرشاة", "قماش", "مثبتات اللوحات",
                "حبر", "رف", "مكتبة", "منضدة", "مرآة المكياج", "عطر", "صندوق مجوهرات", "وسادة", "غطاء",
                "مفارش الطاولة", "أواني", "أدوات الطعام", "أدوات المائدة", "مطبخ", "زجاجة", "كوب القهوة", "كوب الشاي",
                "صحن", "وعاء", "مفارش الطاولة", "منديل", "ستارة الدش", "سخان المناشف", "مرحاض", "حوض", "حوض الاستحمام",
                "دش", "منشفة", "روب الاستحمام", "مجفف الشعر", "مملس الشعر", "فرشاة الشعر", "مكياج", "ظلال العيون",
                "أحمر الشفاه", "مسكارا", "فرشاة المكياج", "عود القطن", "إزالة المكياج", "مرطب", "مزيل العرق", "شامبو",
                "بلسم", "جل الاستحمام", "صابون", "لوشن", "مرطب الشفاه", "مناديل", "كيس القمامة", "سلة الملابس",
                "غسل الملابس", "منعم الأقمشة", "مكواة الملابس", "مكتب المكواة", "مكواة", "مكنسة", "جارف الغبار", "دلو",
                "قماش"]
            randomize = choice(words)
            word = [randomize] * (PlayersNumber-1)
            word.append("imposter")
            for i in range(PlayersNumber):
                player = input("Enter your name:\n>>    ")
                players[i] = player
                identity = choice(word)
                if identity == "imposter":
                    imposter = i
                    word.remove("imposter")
                    pass_the_phone = input("\033[91mYOU ARE THE IMPOSTER\n\033[0mPress enter and pass the phone to another player: ")
                    print("\n" * 4000)
                else:
                    pass_the_phone = input(f"\033[92mYOU ARE A CREWMATE\n\033[0mThe word is {identity}\nPress enter and pass the phone to another player: ")
                    word.remove(word[0])
                    print("\n" * 4000)
            while len(players) > 2 and imposter in players:
                start_voting = input("Press enter when you are going to start voting for the imposter: ")
                print(players, "\n Each Player vote to what he want using player's id!")
                votes = {}
                for i in range(len(players)):
                    id = int(input("\nWhich Player you are going to vote?\n>>    "))
                    if id in votes:
                        votes[id] += 1
                    else:
                        votes[id] = 1
                out = max(votes, key=votes.get)
                found_duplicate = False
                for key1, value1 in votes.items():
                    for key2, value2 in votes.items():
                        if key1 != key2 and value1 == value2:
                            found_duplicate = True
                            print("No player has been kicked from the game")
                            break
                    if found_duplicate:
                        break
                if not found_duplicate:
                    print(players[out], "was kicked from the game")
                    if out == imposter:
                        print(players[out], "is the imposter\n\033[92mCrewmates won!")
                        break
                    else:
                        print(players[out], "is not the imposter")
                        players.pop(out)
            else:
                print(players[imposter], "was the imposter\n\033[91mImposter won!")
        else:
            print("languages availble are ('eng': English / 'fr': French / 'ar': Arabic)")


def limit(func, approaching, side=""):
    k = 1e16
    if approaching == float('inf'):
        approaching = 99999999999
    if approaching == float('-inf'):
        approaching = -99999999999
    if side == "":
        while True:
            try:
                result1 = func(approaching+1/k)
                result2 = func(approaching-1/k)
                break
            except:
                k = k // 10
        if result1 == result2:
            if result1 >= 1e10:
                return float('inf')
            elif result1 <= -1e10:
                return float('-inf')
            return result1
        else:
            raise NameError("Limit doesn't exist")
    elif side == "+":
        while True:
            try:
                result = func(approaching+1/k)
                break
            except:
                k = k // 10
        if result >= 1e10:
            return float("inf")
        elif result <= -1e10:
            return float("-inf")
        return result
    elif side == "-":
        while True:
            try:
                result = func(approaching-1/k)
                break
            except:
                k = k // 10
        if result >= 1e10:
            return float("inf")
        elif result <= -1e10:
            return float("-inf")
        return result
    else:
        raise ValueError("The input for 'side' must be a string type containing only '+' or '-' or nothing.")


def cbrt(x):
    from .cos import cos
    from .sin import sin
    from .constants import pi
    from .sqrt import sqrt
    from .atan import atan
    if type(x) is complex:
        a = x.real
        b = x.imag
        if a != 0:
            r = sqrt(a**2+b**2)
            theta = atan(b/a)
            low = 0.0
            high = max(1.0, r)
            epsilon = 0.00000000000001
            while True:
                mid = (low + high)/2
                mid_cubed = mid * mid * mid

                if abs(mid_cubed-r) < epsilon:
                    cubed_r = mid
                    break
                elif mid_cubed < r:
                    low = mid
                else:
                    high = mid
            sqrt1 = cubed_r * (cos((-pi-theta)/3)-sin((-pi-theta)/3)*1j)
            sqrt2 = cubed_r * (cos((pi-theta)/3)-sin((pi-theta)/3)*1j)
            sqrt3 = -cubed_r * (cos((theta)/3)+sin((theta)/3)*1j)
            if sqrt1.real > sqrt2.real and sqrt1.real > sqrt3.real:
                return sqrt1
            elif sqrt2.real > sqrt1.real and sqrt2.real > sqrt3.real:
                return sqrt2
            return sqrt3 
            
        if b >= 0:
            low = 0.0
            high = max(1.0, b)
        else:
            low = min(-1.0, b)
            high = 0.0
        
        epsilon = 0.00000000000001
    
        while True:
            mid = (low + high) / 2
            mid_cubed = mid * mid * mid

            if abs(mid_cubed - b) < epsilon:
                cubed_b = mid
                break
            elif mid_cubed < b:
                low = mid
            else:
                high = mid
        return cubed_b*-1j
    if x >= 0:
        low = 0.0
        high = max(1.0, x)
    else:
        low = min(-1.0, x)
        high = 0.0
    
    epsilon = 1e-10
    
    while True:
        mid = (low + high) / 2
        mid_cubed = mid * mid * mid

        if abs(mid_cubed - x) < epsilon:
            return mid
        elif mid_cubed < x:
            low = mid
        else:
            high = mid


def ln(x):
    from .integrate import integrate
    from .constants import pi
    from .atan import atan
    from .sqrt import sqrt
    def integrand(t):
        return 1/t
    if type(x) is complex:
        r = sqrt((x.real)**2+(x.imag)**2)
        theta = atan(x.imag/x.real)
        return integrate(integrand, 1, r) + theta*1j
    else:
        if x < 0:
            return integrate(integrand, 1, -x) + pi*1j
        elif x == 0:
            return None
        return integrate(integrand, 1, x)


def mystery():
    from random import randint, choice
    ch = 0
    while ch == 0:
        ch = 1
        select = int(input("---------- 1: Easy Level ----------\n---------- 2: Meduim Level ----------\n---------- 3: Hard Level ----------\nSelect your level:\n>>    "))
        attempt = 0
        if select == 1:
            guess = randint(0, 100)
            data = int(input("Guess the number from 0 -> 100:\n>>    "))
            for i in range(15):
                if data == guess:
                    attempt += 1
                    print("YOU GOT IT IN", attempt, "attempts")
                    break
                elif data > guess:
                    attempt += 1
                    count = 15 - attempt
                    print(count, "attempts left.")
                    data = int(input("less\n>>    "))
                elif data < guess:
                    attempt += 1
                    count = 15 - attempt
                    print(count, "attempts left.")
                    data = int(input("more\n>>    "))
                if attempt == 14:
                    if data != guess:
                        print("Failed :( try again later...")
                        break
        elif select == 2:
            guess = randint(0, 1000)
            data = int(input("Guess the number from 0 -> 1000:\n>>    "))
            for i in range(10):
                if data == guess:
                    attempt += 1
                    print("YOU GOT IT IN", attempt, "attempts")
                    break
                elif data > guess:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    data = int(input("less\n>>    "))
                elif data < guess:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    data = int(input("more\n>>    "))
                if attempt == 9:
                    if data != guess:
                        print("Failed :( try again later...")
                        break
        elif select ==3:
            guess = randint(10, 100)
            chr = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"]
            char = choice(chr)
            data = int(input("Guess the number from 10 -> 100:\n>>    "))
            datach = str(input("Guess the character from a -> z:\n>>    "))
            for i in range(10):
                datach = datach.lower()
                if data == guess and datach == char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("YOU GOT IT IN", attempt, "attempts")
                    break
                elif data == guess and datach > char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is correct. Character is before")
                    datach = str(input("character is before\n>>    "))
                    datach = datach.lower()
                elif data == guess and datach < char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is correct. Character is after")
                    datach = str(input("character is after\n>>    "))
                    datach = datach.lower()
                elif data > guess and datach == char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print ("Character is correct. Number is less")
                    data = int(input("number is less\n>>    "))
                elif data > guess and datach > char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is less. Character is before")
                    data = int(input("number is less\n>>    "))
                    datach = str(input("character is before\n>>    "))
                    datach = datach.lower()
                elif data > guess and datach < char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is less. Character is after")
                    data = int(input("number is less\n>>    "))
                    datach = str(input("character is after\n>>    "))
                    datach = datach.lower()
                elif data < guess and datach == char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Character is correct. Number is more")
                    data = int(input("number is more\n>>    "))
                elif data < guess and datach < char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is more. Character is after")
                    data = int(input("number is more\n>>    "))
                    datach = str(input("character is after\n>>    "))
                    datach = datach.lower()
                elif data < guess and datach > char:
                    attempt += 1
                    count = 10 - attempt
                    print(count, "attempts left.")
                    print("Number is more. Character is before")
                    data = int(input("number is more\n>>    "))
                    datach = str(input("character is before\n>>    "))
                    datach = datach.lower()
                if attempt == 9:
                    if data != guess or datach != char:
                        print("Failed :( try again later...")
                        break
        else:
            print("error at choicing level :(")
            ch = 0


def bin_rep(n):
    from struct import pack
    binary_representation = pack(">d", n)
    binary_representation = "".join(f"{b:08b}" for b in binary_representation)
    bin_rep.sign = binary_representation[0]
    bin_rep.exponent = binary_representation[1:12]
    bin_rep.mantissa = bin_rep.fraction = binary_representation[12:]
    return bin_rep.sign + " " + bin_rep.exponent + " " + bin_rep.mantissa


def cubic(a,b,c,d):
    from .cbrt import cbrt
    from .sqrt import sqrt
    from .quad import quad
    if a == 0:
        raise ValueError("value of 'a' must be different to 0.")
    if d == 0:
        quad(a,b,c)
        cubic.x1, cubic.x2, cubic.x3 = 0, quad.x1, quad.x2
    elif a+b+c+d == 0:
        A = a
        B = b+a
        C = -d
        quad(A,B,C)
        cubic.x1, cubic.x2, cubic.x3 = 1, quad.x1, quad.x2
    elif -a+b-c+d == 0:
        A = a
        B = b-a
        C = d
        quad(A,B,C)
        cubic.x1, cubic.x2, cubic.x3 = -1, quad.x1, quad.x2
    else:
        p = (3*a*c-b**2)/(3*a**2)
        q = (2*b**3-9*a*b*c+27*a**2*d)/(27*a**3)
        cubic.x1 = cbrt(-q/2+sqrt(q**2/4+p**3/27))+cbrt(-q/2-sqrt(q**2/4+p**3/27))-b/(3*a)
        cubic.x2 = (-1+sqrt(-3))/2*cbrt(-q/2+sqrt(q**2/4+p**3/27))+(-1-sqrt(-3))/2*cbrt(-q/2-sqrt(q**2/4+p**3/27))-b/(3*a)
        cubic.x3 = (-1-sqrt(-3))/2*cbrt(-q/2+sqrt(q**2/4+p**3/27))+(-1+sqrt(-3))/2*cbrt(-q/2-sqrt(q**2/4+p**3/27))-b/(3*a)
    if type(cubic.x1) is complex:
        if abs(cubic.x1.imag) < 1e-6:
            cubic.x1 = cubic.x1.real+0j
        if abs(cubic.x1.real) < 1e-6:
            cubic.x1 = 0+cubic.x1.imag*1j
        if cubic.x1.imag == 0:
            cubic.x1 = cubic.x1.real
    if type(cubic.x2) is complex:
        if abs(cubic.x2.imag) < 1e-6:
            cubic.x2 = cubic.x2.real+0j
        if abs(cubic.x2.real) < 1e-6:
            cubic.x2 = 0+cubic.x2.imag*1j
        if cubic.x2.imag == 0:
            cubic.x2 = cubic.x2.real
    if type(cubic.x3) is complex:
        if abs(cubic.x3.imag) < 1e-6:
            cubic.x3 = cubic.x3.real+0j
        if abs(cubic.x3.real) < 1e-6:
            cubic.x3 = 0+cubic.x3.imag*1j
        if cubic.x3.imag == 0:
            cubic.x3 = cubic.x3.real
    if type(cubic.x1) is not complex:
        if abs(cubic.x1) < 1e-6:
            cubic.x1 = 0
    if type(cubic.x2) is not complex:
        if abs(cubic.x2) < 1e-6:
            cubic.x2 = 0
    if type(cubic.x3) is not complex:
        if abs(cubic.x3) < 1e-6:
            cubic.x3 = 0
    pre_result = [cubic.x1,cubic.x2,cubic.x3]
    result = []
    for solution in pre_result:
        if solution not in result:
            result.append(solution)
    return result


def ment_calc():
    from time import sleep
    from threading import Thread
    from random import randint,choice
    t = 1
    print('You must get most points in mental calculation in 30 secondes')
    sleep(5)
    def SpeedRun():
        p = 0
        while t > 0:
            f = randint(0,10)
            s = randint(0,10)
            rand = ['add', 'sub', 'mult', 'div']
            o = choice(rand)
            if o == 'add':
                data = f + s
                ans = int(input(str(f)+' + '+str(s)+' = ?\n>>   '))
                if ans == data:
                    print('Correct!')
                    p += 1
                else:
                    print('Fail!')
            if o == 'sub':
                data = f - s
                ans = int(input(str(f)+' - '+str(s)+' = ?\n>>   '))
                if ans == data:
                    print('Correct!')
                    p += 1
                else:
                    print('Fail!')
            if o == 'mult':
                data = f * s
                ans = int(input(str(f)+' * '+str(s)+' = ?\n>>   '))
                if ans == data:
                    print('Correct!')
                    p += 1
                else:
                    print('Fail!')
            if o == 'div':
                s = randint(1,5)
                if s == 1:
                    f = randint(0,10)
                elif s == 2:
                    choices = [0,2,4,6,8,10]
                    f = choice(choices)
                elif s == 3:
                    choices = [0,3,6,9]
                    f = choice(choices)
                elif s == 4:
                    choices = [0,4,8]
                    f = choice(choices)
                elif s == 5:
                    choices = [0,5,10]
                    f = choice(choices)
                data =  f / s
                ans = int(input(str(f)+' / '+str(s)+' = ?\n>>   '))
                if ans == data:
                    print('Correct!')
                    p += 1
                else:
                    print('Fail!')
        if t == 0:
            print('Time is up!\nYour score is',p)
    Thread(target=SpeedRun).start()
    sleep(30)
    t = 0


def cos(x, deg=False):
    from .constants import pi, e
    if deg:
        x = pi*(x/180)
    if type(x) is not complex:
        return ((e**(1j*x)+e**(1j*-x))/2).real
    if ((e**(1j*x)+e**(1j*-x))/2).imag < 1e-6:
        return ((e**(1j*x)+e**(1j*-x))/2).real
    return (e**(1j*x)+e**(1j*-x))/2


from setuptools import setup, find_packages

setup(
    name="AzizKitten",
    version="11.1.2",
    packages=find_packages(),
    include_package_data=True,
    author="AzizKitten",
    author_email="azizprv43@gmail.com",
    description="AzizOmrane's own python library",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programmig Language :: Python 3.10",
        "Programming Language :: Python 3.11",
        "Programming Language :: Python 3.12",
        "Topic :: Utilities",
    ],
    python_requires=">=3.10"
)




def integrate(integrand, a, b):
    if b == float('inf'):
        b = 999999999999999
    if b == float('-inf'):
        b = -999999999999999
    if a == float('inf'):
        a = 999999999999999
    if a == float('-inf'):
        a = -999999999999999
    segment_width = (b - a) / 60000
    result = 0.5 * (integrand(a) + integrand(b))
    for i in range(1,60000):
        x_i = a + i * segment_width
        result += integrand(x_i)
    result *= segment_width
    if result >= 1e10:
        return float('inf')
    elif result <= -1e10:
        return float('-inf')
    return result


from .mystery import mystery
from .ment_calc import ment_calc
from .sqrt import sqrt
from .quad import quad
from .among_us import among_us
from .bin_rep import bin_rep
from .gcd import gcd
from .lcm import lcm
from .sin import sin
from .cos import cos
from .tan import tan
from .cot import cot
from .sec import sec
from .csc import csc
from .asin import asin
from .acos import acos
from .atan import atan
from .acot import acot
from .asec import asec
from .acsc import acsc
from .sinh import sinh
from .cosh import cosh
from .tanh import tanh
from .coth import coth
from .sech import sech
from .csch import csch
from .floor import floor
from .exp import exp
from .factorial import factorial
from .integrate import integrate
from .limit import limit
from .ln import ln
from .log import log
from .radians import radians
from .degrees import degrees
from .cbrt import cbrt
from .cubic import cubic
from .constants import *


def asin(x):
    from .integrate import integrate
    from .constants import pi
    if -1 <= x <= 1:
        if x == 1:
            return pi/2
        elif x == -1:
            return -(pi/2)
        else:
            def integrand(t):
                return 1/(1-t**2)**.5
            return integrate(integrand, 0, x)
    else:
        raise ValueError("Value input must be in [-1 .. 1]")
